from ReadOnlyDatabase import ReadOnlyDatabase
import shutil
import os.path
import Queue
from Action import Action
from Property import Property
from Sentence import Sentence
from Database import Database
from Link import Link

class DatabaseTest:
    
    def __init__(self):
        
        self.db = Database()
        self.db.createNewDatabase("T:\\temp.db3")
        
        #self.createTest()
        #self.linkTest()
        #self.removeTest()
        self.orderingTest()
        
        
    def createTest(self):
        db = self.db
        pt = self.printTest
        
        sentence = Sentence("eerste zin")
        sentence = self.db.updateSentence(sentence)
        self.printTest("F",sentence.isRootNode())
        self.printTest("[]",self.db.getRootSentences())
        
        property = Property("is_root","1")
        sentence.addProperty(property)
        sentence = self.db.updateSentence(sentence)
        self.printTest("T",sentence.isRootNode())
        self.printTest("[1]",self.db.getRootSentences())
        property = sentence.getProperty("is_root")
        self.printTest("is_root 1",property)
        
        property.setValue("0")
        sentence = self.db.updateSentence(sentence)
        self.printTest("is_root 0",property)
        self.printTest("F",sentence.isRootNode())
        self.printTest("[]",self.db.getRootSentences())
        
        action = Action("first pre action",Action.PREACTION)
        sentence.addPreAction(action)
        sentence = db.updateSentence(sentence)
        pt("[1]",sentence.getPreActionList())
        pt("[]",sentence.getPostActionList())
        pt("Action",sentence.getPreAction(0))
        pt("None",sentence.getPreAction(1))
        
        action = Action("first post action",Action.POSTACTION)
        sentence.addPostAction(action)
        sentence = db.updateSentence(sentence)
        pt("[1]",sentence.getPostActionList())
        pt("[1]",sentence.getPreActionList())
        pt("Action",sentence.getPostAction(0))
        pt("None",sentence.getPostAction(1))
        
        property = Property("misc property","first value")
        sentence.addProperty(property)
        sentence = db.updateSentence(sentence)
        pt("misc property",property)
        pt("misc property",sentence.getProperty("misc property"))
        pt("[2]",sentence.getPropertys())
        
        sentence = db.updateSentence(sentence,True,False)
        pt("misc property",sentence.getProperty("misc property"))
        pt("[2]",sentence.getPropertys())
        
        sentence = db.getSentenceById(sentence.getId(), True, False)
        pt("misc property",sentence.getProperty("misc property"))
        pt("[2]",sentence.getPropertys())
        
        db.showProperty(sentence.getProperty("misc property").getId(), Database.DISABLED)
        sentence = db.updateSentence(sentence,True,False)
        pt("misc property",sentence.getProperty("misc property"))
        pt("[2]",sentence.getPropertys())
        
        sentence = db.updateSentence(sentence)
        pt("None",sentence.getProperty("misc property"))
        pt("[1]",sentence.getPropertys())
        
        sentence = db.updateSentence(sentence,True)
        pt("None",sentence.getProperty("misc property"))
        pt("[1]",sentence.getPropertys())
        
        sentence = db.updateSentence(sentence,True,False)
        db.showProperty(sentence.getProperty("misc property").getId(), Database.ENABLED)
        sentence = db.updateSentence(sentence,True,False)
        pt("misc property",sentence.getProperty("misc property"))
        pt("[2]",sentence.getPropertys())
        
        sentence.getProperty("is_root").setValue("1")
        sentence = db.updateSentence(sentence)
        
        
        
    def linkTest(self):
        db = self.db
        pt = self.printTest
        pt("[x]",db.getRootSentences())
        s1 = Sentence("S1")
        s2 = Sentence("S2")
        s21 = Sentence("S21")
        s22 = Sentence("S22")
        s3 = Sentence("S3")
        s3g = []
        for i in range(10):
            s3g.append(Sentence("S3g"+str(i)))
        
        s1 = db.updateSentence(s1)
                
        s2.addParent(Link(s1.getId(),None))
        
        s2 = db.updateSentence(s2)
        s1 = db.getSentenceById(s1.getId(), True)
        
        pt("[1]",s1.getChildren())
        pt("[]",s1.getParents())
        pt("[]",s2.getChildren())
        pt("[1]",s2.getParents())
        pt("[1]",s1.getChildrenOrdered())
        pt("Link",s1.getChildren()[0])
        
        s21.addParent(Link(s2.getId(),None))
        s22.addParent(Link(s2.getId(),None))
        
        s21 = db.addNewSentence(s21, True)
        s22 = db.updateSentence(s22, True)
        s2 = db.getSentenceById(s2.getId(),True)
        pt("[2]",s2.getChildren())
        pt("[1]",s2.getParents())
        pt("[1]",s21.getParents())
        pt("[1]",s22.getParents())
        pt("[]",s21.getChildren())
        
        pt("s21",s2.getChildren()[0])
        pt("s22",s2.getChildren()[1])
        
        s2.getChildren()[0].setOrder(2)
        s2.getChildren()[1].setOrder(1)
        s2 = db.updateSentence(s2,True)
        
        pt("s22",s2.getChildrenOrdered()[0])
        pt("s21",s2.getChildrenOrdered()[1])
        
        
        s3 = db.updateSentence(s3,True)
        s3.addParent(Link(s22.getId(),s3.getId()))
        s3 = db.updateSentence(s3,True)
        
        s22 = db.getSentenceById(s22.getId())
        pt("[1]",s22.getChildren())
        
        news3g = []
        
        for item in s3g:
            item.addParent(Link(s21.getId(),None,5))
            news3g.append(db.updateSentence(item,True))
        
        s21.addChild(Link(s21.getId(),news3g[5].getId(),2))
        
        pt("[10]",s21.getChildren())
        
        s21 = db.updateSentence(s21, True)
        for child in s21.getChildren():
            pt("s3gx",child)
        pt("loop","-"*20)
        for child in s21.getChildrenOrdered():
            pt("s3gx",child)
        pt("loop","-"*20)
        s21.addChild(Link(s21.getId(),news3g[3].getId(),8))
        for child in s21.getChildrenOrdered():
            pt("s3gx",child)
        pt("loop","-"*20)    
                
        s21 = db.updateSentence(s21, True)
               
        for child in s21.getChildrenOrdered():
            pt("s3gx",child)
        pt("loop","-"*20)
        
        s21.getChildren()[8].setOrder(1)
        for child in s21.getChildrenOrdered():
            pt("s3gx",child)
        pt("loop","-"*20)    
                
        s21 = db.updateSentence(s21, True)
               
        for child in s21.getChildrenOrdered():
            pt("s3gx",child)
        pt("loop","-"*20)
        
        db.removeAllOrphans()
        
        pt("[x]",db.getRootSentences())
              
    def removeTest(self):
        db = self.db
        pt = self.printTest
        
        s1 = Sentence("S1")
        s1.addProperty(Property("to be removed","True"))
        s1.addProperty(Property("is_root","0"))
        s1.addPreAction(Action("to be removed",Action.PREACTION,order=2))
        s1.addPostAction(Action("to be removed",Action.POSTACTION,order=4))
        
        s1 = db.updateSentence(s1,True)
        
        s2 = Sentence("S2")
        s2.addParent(Link(s1.getId(),None))
        
        s2 = db.updateSentence(s2,True)
        s1 = db.getSentenceById(s1.getId(),True)
        
        pt("S1",s1)
        pt("S2",s2)
        pt("[1]",s1.getChildren())
        pt("[1]",s2.getParents())
        pt("[2]",s1.getPropertys())
        pt("[1]",s1.getPreActionList())
        pt("[1]",s1.getPostActionList())
        
        db.removePropertyFrom(s1.getId(), s1.getProperty("to be removed").getId())
        s1 = db.getSentenceById(s1.getId(), True)
        
        pt("[1]",s1.getPropertys())
        
        pt("[4]",db.getAllProperties())  
        db.cleanProperty()
        db.cleanPropertyList()
        
        s1 = db.getSentenceById(s1.getId(),True)
        
        pt("[1]",s1.getPropertys())
                
        db.removeProperty(s1.getProperty("is_root").getId())
        s1 = db.getSentenceById(s1.getId(),True)
        
        pt("[]",s1.getPropertys()) 
        
        pt("[misc property]",db.getAllProperties())  
        
        db.removeActionFrom(s1.getId(), s1.getPreAction(0).getId(),Action.PREACTION)
        
        s1 = db.getSentenceById(s1.getId(), True)
        
        pt("[]",s1.getPreActionList())
        pt("[1]", s1.getPostActionList())
        
        db.removeAction(s1.getPostAction(0).getId())
                
        s1 = db.getSentenceById(s1.getId(),True)
        
        pt("[]",s1.getPostActionList())
        
            
        db.removeSentenceWithoutFloat(s1.getId())         
        
        s1 = db.getSentenceById(s1.getId(),True)
        s2 = db.getSentenceById(s2.getId())
        
        pt("None",s1)
        pt("None",s2)
        
        
        
    def orderingTest(self):
        db = self.db
        pt = self.printTest
        
        sentenceList = []
        actionRList = []
        actionOList = []
        
        for i in range(10):
            sentence = Sentence("S"+str(i))
            action = Action("AR"+str(i),Action.PREACTION,order=i)
            actionRList.append(action)
            for a in actionRList:
                sentence.addPreAction(a)
                
            
            action = Action("AO"+str(i),Action.POSTACTION,order=i)
            actionOList.append(action)
            for a in actionOList:
                sentence.addPostAction(a)
                
            sentenceList.append(db.updateSentence(sentence, True))
        
        for i in range(len(sentenceList)):
            sentence = sentenceList[i]
            pt("["+str(i+1)+"]",sentence.getPreActionList())
            pt("["+str(i+1)+"]",sentence.getPostActionList()) 
            
        sentence = sentenceList[9]
        
        for action in range(len(sentence.getPreActionList())):
            pt("AR"+str(action),sentence.getPreAction(action))
            
        for action in range(len(sentence.getPostActionList())):
            pt("AO"+str(action),sentence.getPostAction(action)) 
        
        sentence.getPreAction(5).setOrder(11)
        sentence.getPostAction(5).setOrder(11)
        
        sentence.getPreAction(4).setOrder(2)
        sentence.getPostAction(4).setOrder(2)
        
        order = [0,1,2,4,3,6,7,8,9,5]
        
        sentence.getPreActionListOrdered()
        sentence.getPostActionListOrdered()
        
        counter = 0
        for action in order:
            pt("AR"+str(action),sentence.getPreAction(counter))
            pt("AO"+str(action),sentence.getPostAction(counter))
               
            counter += 1
        
        sentence.addChild(Link(sentence.getId(),sentenceList[0].getId(),order=2))
        sentence.addChild(Link(sentence.getId(),sentenceList[1].getId(),order=1))
        sentence.addChild(Link(sentence.getId(),sentenceList[2].getId(),order=4))
        sentence.addChild(Link(sentence.getId(),sentenceList[3].getId(),order=3))
        
        sentence = db.updateSentence(sentence, True)
        
        childs = sentence.getChildrenOrdered()
        
        pt("S1",childs[0])
        pt("S0",childs[1])
        pt("S3",childs[2])
        pt("S2",childs[3])
            
        for sentence in sentenceList:   
            db.removeSentence(sentence.getId())
        
        db.cleanAction()   
                
    
    def printTest(self,condition,result):
        print str(condition)+"| "+str(result)

DatabaseTest()